var outputArea = document.getElementById("output");
var input = document.getElementById("filenames");
var outputDiv = document.getElementById("outputDiv");
var currentText = "Test";

function copyStringToClipboard(string) 
{ 
	//window.alert(string);
	document.getElementById("dummy").value = string;
	document.getElementById("dummy").focus();
	document.execCommand('SelectAll');
	document.execCommand("Copy", false, null); 
	window.getSelection().removeAllRanges();
}


// Helper function to create a text node in which some html-sequences have been replaced.
function transform(text)
{
	text = text.replaceAll("&lt;","<");
	text = text.replaceAll("&gt;",">");
	
	// In Katex, some symbols are doubly escaped.
	text = text.replaceAll("&amp;amp;","&");
	text = text.replaceAll("&amp;lt;","<");
	text = text.replaceAll("&amp;gt;",">");	
	
	// Remove some useless html code
	text = text.replaceAll("<div>","");	
	text = text.replaceAll("</div>","");	
	text = text.replaceAll("<br>"," ");
	
	return text + "  ";
}

function textNode(text)
{
	text = transform(text);
	return document.createTextNode(text);
}	

// Helper function to create a list of criteria. It depends heavily on the formatting of the criteria in the files to be parsed.
function parseCriteria(text)
{
	spanParse = document.createElement("span");
	ul = document.createElement("ul");
	spanParse.appendChild(ul);
		
	while(text.indexOf("[Criterium]:")==0)
	{
		li = document.createElement("li");
		
		//console.log(text);
		part = text.substr(13);
		ind = part.indexOf("[Criterium]:");
		text = part.substr(ind);

		//console.log(textbb  + "\n\n");		
		if(ind > 0)
		part = part.substr(0, ind);
		
		// Removing the separator if it is present
		if(part.indexOf("-----") > 0)
		{
			part = part.substring(0, part.indexOf("-----") - 1);
		}		

		// Obtain the criterion and the points
		ind = part.indexOf("[points]:");
		points = part.substr(ind);
		first = part.substr(0, ind);
		//console.log(first + " [ " + points.substr(10) + " points ]");
	
		msgParse = textNode(first + " [ " + points.substr(10) + " points ]");
		li.appendChild(msgParse);
		
		if(document.getElementById("includeButtonsToCopy").checked)
		{
			let btn = document.createElement("button");
			//console.log("copyStringToClipboard('" + transform(first).replaceAll("\\","\\\\").replaceAll("'", "\\\'") + "')");
			btn.setAttribute("onclick", "copyStringToClipboard('" + transform(first).replaceAll("\\","\\\\").replaceAll("'", "\\\'") + "')");
			btn.appendChild(document.createTextNode("Copy"));
			li.appendChild(btn);
		}
		ul.appendChild(li);
	}	
	outputDiv.appendChild(spanParse);
}	

async function parseStuff(text)
{
	parser = new DOMParser();
	xmlDoc = parser.parseFromString(text, "text/xml");

	if(xmlDoc.getElementsByTagName("assessmentItem").length == 0)
	{
		return;
	}
		
	output = xmlDoc.getElementsByTagName("assessmentItem")[0].getAttribute("title")
	spanParse = document.createElement("h2");
	msgParse = document.createTextNode(output);
	spanParse.appendChild(msgParse);
	outputDiv.appendChild(spanParse);	

	if(xmlDoc.getElementsByTagName("itemBody")[0].getElementsByTagName("p").length > 0)
	{		
		spanParse = document.createElement("h3");
		msgParse = document.createTextNode("Description");
		spanParse.appendChild(msgParse);
		outputDiv.appendChild(spanParse);	
	
		output = xmlDoc.getElementsByTagName("itemBody")[0].getElementsByTagName("p")[0].getInnerHTML()
		//console.log(output);
	
		spanParse = document.createElement("span");
		output = output.substr(92,output.length-116) // Remove the <div ...> and </div>
		msgParse = textNode(output);
		spanParse.appendChild(msgParse);
		
		if(document.getElementById("includeButtonsToCopy").checked)
		{
			let btn = document.createElement("button");
			//console.log("copyStringToClipboard('" + transform(output).replaceAll("\\","\\\\").replaceAll("'", "\\\'") + "')");
			btn.setAttribute("onclick", "copyStringToClipboard('" + transform(output).replaceAll("\\","\\\\").replaceAll("'", "\\\'") + "')");
			btn.appendChild(document.createTextNode("Copy"));
			spanParse.appendChild(btn);
		}
		outputDiv.appendChild(spanParse);
	}
	if(xmlDoc.getElementsByTagName("itemBody")[0].getElementsByTagName("div").length > 0)
	{
		spanParse = document.createElement("h3");
		msgParse = document.createTextNode("Question");
		spanParse.appendChild(msgParse);
		outputDiv.appendChild(spanParse);	

		output = xmlDoc.getElementsByTagName("itemBody")[0].getElementsByTagName("div")[0].getInnerHTML()
		//console.log(output);
		
		spanParse = document.createElement("span");
		msgParse = textNode(output); // Remove the <div> and </div>
		spanParse.appendChild(msgParse);

		if(document.getElementById("includeButtonsToCopy").checked)
		{
			let btn = document.createElement("button");
			//console.log("copyStringToClipboard('" + transform(output).replaceAll("\\","\\\\").replaceAll("'", "\\\'") + "')");
			btn.setAttribute("onclick", "copyStringToClipboard('" + transform(output).replaceAll("\\","\\\\").replaceAll("'", "\\\'") + "')");
			btn.appendChild(document.createTextNode("Copy"));
			spanParse.appendChild(btn);
		}
		outputDiv.appendChild(spanParse);
	}

	if(xmlDoc.getElementsByTagName("rubricBlock").length > 0)
	{
		spanParse = document.createElement("h3");
		msgParse = document.createTextNode("Criteria");
		spanParse.appendChild(msgParse);
    		outputDiv.appendChild(spanParse);	

		output = xmlDoc.getElementsByTagName("rubricBlock")[0].getInnerHTML()
		//console.log(output);

		parseCriteria(output);
	}
}

function readXmlFile(i)
{
	var content = input.files[i];
	let reader = new FileReader();
	reader.readAsText(content);
	
	//console.log(input.files[i].name);
	
	reader.onload = function()
	{
		text = reader.result;
		parseStuff(text);
	};
	reader.onerror = function() 
	{
		console.log(reader.error);
	};
}

async function readZipFile(file)
{
	outputArea.value = outputArea.value + "We start unzipping the following file: " + file.name + "\n";

	entries = await new zip.ZipReader(new zip.BlobReader(file)).getEntries();
	j = 0;
	while( j < entries.length)
	{
		entry = entries[j];
		outputArea.value = outputArea.value + "- Parsing a zipped file: " + String.fromCharCode.apply(null, entry.rawFilename) + " \n";
		fileContent = await entry.getData(new zip.TextWriter())	
		//console.log(fileContent);		

		parseStuff(fileContent);
		j = j  + 1;
	}

	//console.log(twan);	
	
}

function processFiles()
{
	// Remove all previous information form outputDiv
	while (outputDiv.lastChild) 
	{
		outputDiv.removeChild(outputDiv.lastChild);
    	}
	outputArea.value = "We try to parse the following files:\n";
	
	if(document.getElementById("includeButtonsToCopy").checked)
	{
		outputDiv.setAttribute("style", "height:400px;overflow:scroll");
	}
	else
	{
		outputDiv.setAttribute("style", "");
	}

	// Try to parse all the files that have been selected
	i = 0;
	while(i < input.files.length)
	{
		
		if(input.files[i].name.endsWith(".zip"))
		{
			outputArea.value = outputArea.value + "Unzipping " + input.files[i].name + "\n";
			readZipFile(input.files[i]);
			//return;
		}
		
		if(input.files[i].name.endsWith("xml"))
		{
			outputArea.value = outputArea.value + "Parsing " + input.files[i].name + "\n";
			readXmlFile(i);
		}
		i=i+1;
	}
	input.value = null;
}


		